<?php
declare(strict_types=1);

namespace Inti\Provider;

use Inti\Provider\Query\QueryInterface;
use Inti\Provider\Query\QueryTrait;
use RuntimeException;

use function array_filter;
use function array_key_exists;
use function count;
use function intval;
use function sprintf;

/**
 * Class AbstractProvider
 *
 * @package Inti\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
abstract class AbstractProvider implements
    ProviderInterface,
    QueryInterface
{
    use QueryTrait;

    protected string $idField = 'id';

    protected function fetchResults(): array
    {
        return [];
    }

    protected function filter(array $result): array
    {
        return $this->filterResult($result);
    }

    protected function filterResult(array $result): array
    {
        return $result;
    }

    protected function getId(array $item): int
    {
        if (!array_key_exists($this->idField, $item)) {
            throw new RuntimeException(sprintf(
                "%s: Item array does not have a key '%s'.",
                __METHOD__, $this->idField
            ));
        }

        return intval($item[$this->idField]);
    }

    public function getCount(): int
    {
        return count($this->getItems());
    }

    public function getItem(): ?array
    {
        return $this->getItems()[0] ?? null;
    }

    public function getItems(): array
    {
        $results = $this->fetchResults();
        return array_filter([$this, 'filter'], $results);
    }
}
